# Esercizio Tripartizione (PNL)

# DATI
param nS;						# Numero di sottinsiemi
set S := 0..nS-1;				# Insieme dei sottinsiemi
param nP {S};					# Numero di punti in ogni sottinsieme
set P {s in S} := 1..nP[s];		# Insieme dei punti in ogni sottinsieme
param x {s in S, P[s]};			# Ascissa di ogni punto
param y {s in S, P[s]};			# Ordinata di ogni punto

# VARIABILI
# Posizione del punto di origine
var xx := 0;
var yy := 0;
# Coefficienti delle rette
var a {S};
var b {S};
var c {S};

# VINCOLI
subject to Normalizzazione {s in S}:
  a[s]^2 + b[s]^2 = 1;

# Partizione dei punti
subject to Dx {s in S,p in P[s]}:
  a[s]*x[s,p] + b[s]*y[s,p] + c[s] >= 0;
subject to Sx {s in S,p in P[(s+1) mod nS]}:
  a[s]*x[(s+1) mod nS,p] + b[s]*y[(s+1) mod nS,p] + c[s] <= 0;

# Passaggio per origine
subject to Passaggio {s in S}:
  a[s]*xx + b[s]*yy + c[s] = 0;

# OBIETTIVO
minimize z1: xx^2 + yy^2;

########################
data;
########################

param nS := 3;

param nP :=
0	8
1	6
2	12;

param x (tr):	0	1	2 :=
1				-5	-10	-3
2				10	-4	-1
3				5	-19	1
4				3	-11	-2
5				-1	-7	11
6				-2	-12	2
7				2	.	3
8				10	.	0
9				.	.	1
10				.	.	2
11				.	.	-3
12				.	.	-4;

param y (tr):	0	1	2 :=
1				12	2	-6
2				8	3	-8
3				5	15	-5
4				0	0	-9
5				3	-3	-1
6				9	-9	-9
7				10	.	0
8				6	.	-2
9				.	.	-8
10				.	.	-2
11				.	.	-6
12				.	.	-9;

param: 	a		b		c :=
0		0.7    -0.7	    0
1		0		1		0
2	   -0.7    -0.7	    0;

# Soluzione ottima calcolata da Snopt
#   xx = yy = 0		z1 = 0
#
# Rette   a              b            c
# 0    0.923002       0.384794    1.99897e-08
# 1   -0.601432       0.798924    1.00083e-08
# 2    7.31472e-09   -1          -2.19442e-08

end;
