# Esercizio Tripartizione (PNL)

# DATI
param nS;						# Numero di sottinsiemi
set S := 0..nS-1;				# Insieme dei sottinsiemi
param nP {S};					# Numero di punti in ogni sottinsieme
set P {s in S} := 1..nP[s];		# Insieme dei punti in ogni sottinsieme
param x {s in S, P[s]};			# Ascissa di ogni punto
param y {s in S, P[s]};			# Ordinata di ogni punto

# VARIABILI
# Posizione del punto di origine
var xx := 0;
var yy := 0;
# Direzioni centrali di ogni cluster
var alfa {S} >=0, <=6.28;
# Prodotto scalare tra ogni punto ed ogni direzione centrale
var ps {k in S,h in S,i in P[h]} = 
  ((x[h,i]-xx)*cos(alfa[k]) + (y[h,i]-yy)*sin(alfa[k])) / sqrt((x[h,i]-xx)^2+(y[h,i]-yy)^2);

# VINCOLI
# Separazione tra i clusters:
# il prodotto scalare tra una direzione centrale 
# e i versori dei punti (i) del suo cluster (k)
# deve essere maggiore del suo prodotto scalare 
# con i versori dei punti (i) degli altri clusters (h)
subject to Cluster {k in S, h in S, i in P[k], j in P[h]: h<>k}:
  ps[k,k,i] >= ps[k,h,j];

# OBIETTIVO
minimize z1: xx^2 + yy^2;

########################
data;
########################

param nS := 3;

param nP :=
0	 8
1	 6
2	12;

param x (tr):	0	1	2 :=
1				-5	-10	-3
2				10	-4	-1
3				5	-19	1
4				3	-11	-2
5				-1	-7	11
6				-2	-12	2
7				2	.	3
8				10	.	0
9				.	.	1
10				.	.	2
11				.	.	-3
12				.	.	-4;

param y (tr):	0	1	2 :=
1				12	2	-6
2				8	3	-8
3				5	15	-5
4				0	0	-9
5				3	-3	-1
6				9	-9	-9
7				10	.	-1
8				6	.	-2
9				.	.	-8
10				.	.	-2
11				.	.	-6
12				.	.	-9;

var alfa :=
0	1.57
1	3.80
2	5.50;

end;

Soluzione ottima di Snopt

xx = -2.06054e-08
yy = 2.62585e-08

alfa [*] :=
0  1.22685
1  3.3285
2  5.26141
