# Esercizio Tripartizione (PNL)

# DATI
param nS;						# Numero di sottinsiemi
set S := 0..nS-1;				# Insieme dei sottinsiemi
param nP {S};					# Numero di punti in ogni sottinsieme
set P {s in S} := 1..nP[s];		# Insieme dei punti in ogni sottinsieme
param x {s in S, P[s]};			# Ascissa di ogni punto
param y {s in S, P[s]};			# Ordinata di ogni punto

# VARIABILI
# Posizione del punto di origine
var xx := 4;
var yy := 4;
# Direzioni centrali di ogni cluster
var alfa {S} >=0, <=6.28;
# Prodotto scalare tra ogni punto ed ogni direzione centrale
var ps {k in S,h in S,i in P[h]} = 
  ((x[h,i]-xx)*cos(alfa[k]) + (y[h,i]-yy)*sin(alfa[k])) / sqrt((x[h,i]-xx)^2+(y[h,i]-yy)^2);
# Minima differenza tra prodotti scalari
var delta;

# VINCOLI
# Separazione tra i clusters:
# il prodotto scalare tra una direzione centrale 
# e i versori dei punti (i) del suo cluster (k)
# deve essere maggiore del suo prodotto scalare 
# con i versori dei punti (i) degli altri clusters (h)
subject to Cluster {k in S, h in S, i in P[k], j in P[h]: h<>k}:
  ps[k,k,i] >= ps[k,h,j] + delta;

# OBIETTIVO
maximize z1: delta;

########################
data;
########################

param nS := 3;

param nP :=
0	3
1	1
2	1;

param x (tr):	0	1	2 :=
1				0	2	6
2				4	.	.
3				8	.	.;

param y (tr):	0	1	2 :=
1				0	1	1
2				6	.	.
3				0	.	.;

var alfa :=
0	1.57
1	3.80
2	5.50;

end;
